﻿using System;
using System.Collections.Generic;
using ThinkGeo.MapSuite.Core;
using System.Collections.ObjectModel;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace Post
{
    public class SelectExtentInteractiveOverlay : ExtentInteractiveOverlay
    {
        public SelectExtentInteractiveOverlay(FeatureLayer sourceFeatureLayer)
        {
            SourceFeatureLayer = sourceFeatureLayer;
        }

        public FeatureLayer SourceFeatureLayer { get; set; }

        public event EventHandler<SelectedSelectExtentInteractiveOverlayEventArgs> Selected;

        protected override InteractiveResult MouseUpCore(InteractionArguments interactionArguments)
        {
            InteractiveResult result = base.MouseUpCore(interactionArguments);
            if ((ExtentChangedType == ExtentChangedType.TrackZoomIn || ExtentChangedType == ExtentChangedType.TrackZoomOut) && SourceFeatureLayer != null)
            {
                RectangleShape targetExtent = result.NewCurrentExtent;
                result.NewCurrentExtent = null;

                if (!SourceFeatureLayer.IsOpen) SourceFeatureLayer.Open();
                Collection<Feature> queriedFeatures = SourceFeatureLayer.QueryTools.GetFeaturesInsideBoundingBox(targetExtent, ReturningColumnsType.AllColumns);
                OnSelected(new SelectedSelectExtentInteractiveOverlayEventArgs(queriedFeatures));
            }

            return result;
        }

        protected virtual void OnSelected(SelectedSelectExtentInteractiveOverlayEventArgs e)
        {
            EventHandler<SelectedSelectExtentInteractiveOverlayEventArgs> handler = Selected;
            if (handler != null)
            {
                handler(this, e);
            }
        }
    }

    public class SelectedSelectExtentInteractiveOverlayEventArgs : EventArgs
    {
        private Collection<Feature> selectedFeatures;

        public SelectedSelectExtentInteractiveOverlayEventArgs(IEnumerable<Feature> selectedFeatures)
        {
            this.selectedFeatures = new Collection<Feature>();
            foreach (Feature feature in selectedFeatures)
            {
                this.selectedFeatures.Add(feature);
            }
        }

        public Collection<Feature> SelectedFeatures { get { return selectedFeatures; } }
    }
}
